#
# spec file for package suse-module-tools
#
# Copyright (c) 2018 SUSE LINUX GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


Name:           suse-module-tools
Version:        15.0.1
Release:        lp150.2.3.1
Summary:        Configuration for module loading and SUSE-specific utilities for KMPs
License:        GPL-2.0-or-later
Group:          System/Base
Url:            https://github.com/openSUSE/suse-module-tools
Source0:        %{name}-%{version}.tar.xz
Source1:        %{name}.rpmlintrc
Requires:       /sbin/mkinitrd
Requires:       binutils
Requires:       coreutils
Requires:       findutils
Requires:       grep
Requires:       gzip
# module-init-tools in older distros, kmod-compat in later ones
Requires:       /sbin/depmod
Requires:       rpm
Requires:       sed

%description
This package contains helper scripts for KMP installation and
uninstallation, as well as default configuration files for depmod and
modprobe. These utilities are provided by kmod-compat or
module-init-tools, whichever implementation you choose to install.

%prep
%setup -q

%build
%if 0%{?is_opensuse} == 0
sed -ri 's/^( *allow_unsupported_modules *) 1/\1 0/' \
	10-unsupported-modules.conf
%endif

%install
# now assemble the parts for modprobe.conf
cp modprobe.conf/modprobe.conf.common 00-system.conf
if [ -f "modprobe.conf/modprobe.conf.$RPM_ARCH" ]; then
	cat "modprobe.conf/modprobe.conf.$RPM_ARCH" >>00-system.conf
fi
install -d -m 755 "%{buildroot}%{_sysconfdir}/modprobe.d"
install -pm644 "10-unsupported-modules.conf" \
	"%{buildroot}%{_sysconfdir}/modprobe.d/"
install -pm644 00-system.conf "%{buildroot}%{_sysconfdir}/modprobe.d/"
install -pm644 modprobe.conf/modprobe.conf.local "%{buildroot}%{_sysconfdir}/modprobe.d/99-local.conf"
install -d -m 755 "%{buildroot}%{_sysconfdir}/depmod.d"
install -pm 644 "depmod-00-system.conf" \
	"%{buildroot}%{_sysconfdir}/depmod.d/00-system.conf"

# "module-init-tools" name hardcoded in KMPs, mkinitrd, etc.
install -d -m 755 "%{buildroot}%{_libexecdir}/module-init-tools"
install -pm 755 weak-modules{,2} "%{buildroot}%{_libexecdir}/module-init-tools/"
install -pm 755 driver-check.sh "%{buildroot}%{_libexecdir}/module-init-tools/"

# rpm macros and helper
install -d -m 755 "%{buildroot}%{_sysconfdir}/rpm"
install -pm 644 "macros.initrd" "%{buildroot}%{_sysconfdir}/rpm/"
install -pm 755 "regenerate-initrd-posttrans" "%{buildroot}%{_libexecdir}/module-init-tools/"

# modsign-verify for verifying module signatures
install -d -m 755 "%{buildroot}%{_prefix}/bin"
install -pm 755 modsign-verify "%{buildroot}%{_bindir}/"
install -pm 755 kmp-install "%{buildroot}%{_bindir}/"
# modhash for calculating hash of signed kernel module
install -pm 755 modhash "%{buildroot}%{_bindir}/"

# systemd service to load /boot/sysctl.conf-`uname -r`
install -d -m 755 "%{buildroot}%{_libexecdir}/systemd/system/systemd-sysctl.service.d"
install -pm 644 50-kernel-uname_r.conf "%{buildroot}%{_libexecdir}/systemd/system/systemd-sysctl.service.d"

# Ensure that the sg driver is loaded early (bsc#1036463)
# Not needed in SLE11, where sg is loaded via udev rule.
install -d -m 755 "%{buildroot}%{_sysconfdir}/modules-load.d"
install -pm 644 sg.conf "%{buildroot}%{_sysconfdir}/modules-load.d"

mkdir -p %{buildroot}%{_defaultlicensedir}

%post
%if 0%{?sle_version} >= 150000
# Delete obsolete unsupported-modules file from SLE11
rm -f %{_sysconfdir}/modprobe.d/unsupported-modules
%else
# Logic for releases below CODE 15
%if 0%{?is_opensuse} == 1
allowed=1
%else
allowed=0
%endif
test_allow_on_install()
{
	# configure handling of unsupported modules
	# default is to allow them
	allow=1
	# if the obsolete LOAD_UNSUPPORTED_MODULES_AUTOMATICALLY variable is
	# set to no, don't allow (this was used in SLES 9 and 10)
	if test -e %{_sysconfdir}/sysconfig/hardware/config; then
		. %{_sysconfdir}/sysconfig/hardware/config
		if test "x$LOAD_UNSUPPORTED_MODULES_AUTOMATICALLY" = "xno"; then
			allow=0
		fi
		# obsolete
		rm %{_sysconfdir}/sysconfig/hardware/config
	fi
	# don't change the setting during upgrade
	if test "$1" != 1; then
		allow=
		return
	fi
	# on SLES, the default is not to allow unsupported modules
	if grep -qs "Enterprise Server" %{_sysconfdir}/os-release; then
		allow=0
	else
		return
	fi
	# unless the admin passed "oem-modules=1" to the kernel during install
	if grep -qs '\<oem-modules=1\>' /proc/cmdline; then
		allow=1
		return
	fi
	# or if the installer already loaded some unsupported modules
	# (see TAINT_NO_SUPPORT in /etc/src/linux/include/linux/kernel.h)
	tainted=$(cat /proc/sys/kernel/tainted 2>/dev/null || echo 0)
	if test $((tainted & (1<<30))) != 0; then
		allow=1
		return
	fi
}
# upgrade from old locations
if test -e %{_sysconfdir}/modprobe.d/unsupported-modules; then
	mv -f %{_sysconfdir}/modprobe.d/unsupported-modules \
		%{_sysconfdir}/modprobe.d/10-unsupported-modules.conf
fi
test_allow_on_install "$@"
if test -n "$allow" -a "$allow" != "$allowed"; then
	sed -ri 's/^( *allow_unsupported_modules *) [01]/\1 '"$allow"'/' \
		%{_sysconfdir}/modprobe.d/10-unsupported-modules.conf
fi
%endif

# upgrade from old locations
if test -e %{_sysconfdir}/modprobe.conf.local; then
	mv -f %{_sysconfdir}/modprobe.conf.local \
		%{_sysconfdir}/modprobe.d/99-local.conf
fi

%files
%defattr(-,root,root)

%if 0%{?sle_version:%{sle_version}}%{!?sle_version:150000} <= 120200
%dir %{_defaultlicensedir}
%endif
%license LICENSE
%doc README.SUSE
%dir %{_sysconfdir}/modprobe.d
%config %{_sysconfdir}/modprobe.d/00-system.conf
%config(noreplace) %{_sysconfdir}/modprobe.d/10-unsupported-modules.conf
%config(noreplace) %{_sysconfdir}/modprobe.d/99-local.conf
%dir %{_sysconfdir}/depmod.d
%config %{_sysconfdir}/depmod.d/00-system.conf
%config %{_sysconfdir}/rpm/macros.initrd
%{_bindir}/modhash
%{_bindir}/kmp-install
%{_bindir}/modsign-verify
%{_libexecdir}/module-init-tools
%{_libexecdir}/systemd/system/systemd-sysctl.service.d
%dir %{_sysconfdir}/modules-load.d
%config(noreplace) %{_sysconfdir}/modules-load.d/sg.conf

%changelog
* Wed Nov  7 2018 mwilck@suse.com
- Update to version 15.0.1 (git 5c63319):
- Use /etc/modules-load.d/sg.conf for sg driver autoloading
  (bsc#1036463)
  * added /etc/modules-load.d/sg.conf
* Mon Nov  5 2018 mwilck@suse.com
- version 15.0 (git 6ce3011)
- modsign-verify: support for parsing PKCS#7 signatures
  (bsc#1111300, bsc#1105495)
- Fix logic for unsupported modules (bsc#1111183)
  * SLES15: allow unsupported modules only if kernel-default-extra
  from SLE-WE module is installed
  * Always allow unsupported modules on Leap 15 and Factory
  * No change for SLE12 / Leap 42.x and below
  * 10-unsupported-modules.conf: improved comments
  * fix %%config(noreplace) logic for 10-unsupported-modules.conf
- added udev rule 81-sg.rules to make sure the sg module is
  always loaded (bsc#1036463)
* Wed Jul 26 2017 tchvatal@suse.com
- Use path dependency that is not parsed by obs to ensure we
  are not pulling systemd everywhere
* Mon Jun 19 2017 tchvatal@suse.com
- Set url to github project and repack using obs service
- Cleanup a bit with spec-cleaner
- Grep os-release and not deprecated SuSE-release
* Thu Sep  8 2016 dmueller@suse.com
- blacklist efifb on aarch64 (bsc#996080)
* Fri May 27 2016 mmarek@suse.cz
- Run dos2unix on the modhash script.
* Thu May 26 2016 jlee@suse.com
- Add modhash tool to calculate hash of signed module.
  It strips X.509 or PKCS#7 signature before hash kernel module.
  (fate#319460)
* Wed May 25 2016 mmarek@suse.cz
- Remove -x bit from 50-kernel-uname_r.conf (bsc#981291).
* Fri Apr 29 2016 mmarek@suse.cz
- Ignore failures when loading the kernel sysctl (bsc#977715).
* Thu Apr 28 2016 mmarek@suse.com
- Add a systemd service to load /boot/sysctl.conf-`uname -r`
  (bsc#977715).
* Fri Apr  1 2016 mmarek@suse.cz
- Skip KMPs built from the kernel spec file directly (fate#319339)
- Bump version number to 12.4
* Tue Feb 23 2016 lnussel@suse.de
- add missing requirements on module-init-tools, findutils and gzip,
  mkinitrd (bnc#965830)
* Fri Jan 15 2016 bpoirier@suse.com
- Add module option ib_mthca.catas_reset_disable=1 on ppc64le (bsc#456389).
* Tue Oct 13 2015 mmarek@suse.cz
- Revert change for fate#319339, as some 3rd party KMPs are built
  with a modified template (bsc#949862).
* Tue Sep  8 2015 trenn@suse.de
- Use /run instead of deprecated /var/run
  This can have sever consequences if this package uses /var/run. It may
  be installed before filesystem package generating the /run tmpfs and
  predict it to generate the link /var/run -> /run. Later dracut convertfs
  kicks in to generate it, but it has issues. bsc#922676
* Thu Aug 13 2015 mmarek@suse.cz
- Only handle KMPs that are built using the %%kernel_module_package
  macro (preparatory work for fate#319339).
* Tue Jun 23 2015 mmarek@suse.cz
- Use absolute path to mkinitrd (bsc#935566)
* Fri May 16 2014 mmarek@suse.cz
- Add kgraft/ to the depmod search list (fate#313296)
* Mon May  5 2014 tonyj@suse.com
- Add support for ARM zImage to weak-modules2
* Tue Apr 15 2014 mmarek@suse.cz
- weak-modules2: Use lsinitrd to list the modules in an initrd, so
  that initrds with early microcode work (bnc#873209, bnc#873113).
* Fri Apr  4 2014 mmarek@suse.cz
- weak-modules2: Run mkinitrd in posttrans if $INITRD_IN_POSTTRANS
  is set (fate#313506)
* Fri Apr  4 2014 mmarek@suse.cz
- regenerate-initrd-posttrans: Run mkinitrd_setup
* Fri Apr  4 2014 mmarek@suse.cz
- Add macros for regenerating the initrd in %%posttrans
  (fate#313506).
* Mon Mar 31 2014 mmarek@suse.cz
- Use softdep to order {u,o,e}hci-hcd drivers.
- Drop ia64 modprobe config.
* Fri Mar 28 2014 mmarek@suse.cz
- Add kmp-install tool for easier installation of SolidDriver KMPs
  (fate#314581).
* Tue Mar 11 2014 mmarek@suse.cz
- Add modsign-verify tool to verify signatures of modules
  (fate#314507).
* Tue Mar 11 2014 mmarek@suse.cz
- weak-modules2: Support XZ compressed initrds (bnc#778119,
  bnc#867312)
* Mon Mar 10 2014 mmarek@suse.cz
- The package cannot be noarch, because it installs different
  configs for different architectures.
- Install 10-unsupported-modules.conf unconditionally, kmod has
  been patched.
* Fri Feb  7 2014 rsalevsky@suse.com
- load the uas driver simulsimultaneously with the usb_storage driver and vice versa (bnc#862397)
* Wed May 29 2013 schwab@suse.de
- weak-modules{,2}: also look for Image-$krel for aarch64
* Sun Mar 17 2013 crrodriguez@opensuse.org
- The "allow_unsupported_modules" feature, does not belong too
  openSUSE, kmod (and hence udev and systemd) do not understand
  it in anyway. limit to SLE only.
* Wed Nov 14 2012 jengelh@inai.de
- New package. Split SUSE-specific scripts from module-init-tools,
  so that we can actually get a system with kmod-compat running.
